clear all;close all;clc %#ok<*CLALL>
rng('default')

% description of events that happened during the experiment
FILE_EVENTS = 'events.xlsx';
% data exported from Tobii software
FILE_TOBII = 'data_tobii.tsv';
% questionnaire filled out by the participants
FILE_PP = 'data_pp.xlsx';

%% Import questionnaire data
[raw,~,raw2]=xlsread(FILE_PP,1,'A2:D45'); % read data from questionnaire
xlsPP=raw(:,1); % IDs of participants
xlsAge=raw(:,3); % ages of participants
xlsGender=strcmp(raw2(:,4),'Female')+2*strcmp(raw2(:,4),'Male'); % gender of participants
[SF,HM,Gender,Age,NF,AD,LF,WP]=deal(NaN(44,1));
[AA,AAs]=deal(NaN(44,34));
[NFt,SFDt]=deal(NaN(44,43));

%% Import data on events and gaze samples
% description of occured events
Events=xlsread(FILE_EVENTS,'Events','B4:L47');
% occupance of parking places
Occupancy=xlsread(FILE_EVENTS,'Parking space occupancy','C3:AT59');
% data samples info
Samples=xlsread(FILE_EVENTS,'Time and % samples','D2:D45');

%% Description of events
% events as coded
% 1 = Recording time
% 2 = Participant number
% 3 = Recording date (UTC)
% 4 = Recording time (UTC)
% 5 = Event
% 6 = Gaze point X
% 7 = Gaze point Y
% 8 = Gaze point 3D X
% 9 = Gaze point 3D Y
% 10 = Gaze point 3D Z
% 11 = Gaze direction left X
% 12 = Gaze direction left Y
% 13 = Gaze direction left Z
% 14 = Gaze direction right X
% 15 = Gaze direction right Y
% 16 = Gaze direction right Z
% 17 = Pupil position left X
% 18 = Pupil position left Y
% 19 = Pupil position left Z
% 20 = Pupil position right X
% 21 = Pupil position right Y
% 22 = Pupil position right Z
% 23 = Pupil diameter left
% 24 = Pupil diameter right
% 25 = Gaze event duration
% 26 = Eye movement type index
% 27 = Fixation point X
% 28 = Fixation point Y
% 29 = Gyro X
% 30 = Gyro Y
% 31 = Gyro Z
% 32 = Accelerometer X
% 33 = Accelerometer Y
% 34 = Accelerometer Z

% events in Tobii data
Events_tobii={
    '                                                                  ' % 1
    'Any other moving car                                              ' % 2  (1)
    'Approaching car                                                   ' % 3  (2)
    'Back of  car                                                      ' % 4  (3)
    'Environment (windows, doors, pillars, bins, carts)                ' % 5  (4)
    'Experimenter in the car                                           ' % 6  (5)
    'Experimenter outside the car                                      ' % 7  (6)
    'Front of stationary car                                           ' % 8  (7)
    'Ground (asphalt / lines)                                          ' % 9  (8)
    'Human in any other moving car                                     ' % 10 (9)
    'Human in car (only when human is visible, not any approaching car)' % 11 (10)
    'Human in parking/unparking car                                    ' % 12 (11)
    'Human outside of car                                              ' % 13 (12)
    'Parking car                                                       ' % 14 (13)
    'RecordingEnd                                                      ' % 15 
    'RecordingPause                                                    ' % 16
    'RecordingStart                                                    ' % 17
    'Side of car                                                       ' % 18 (14)
    'SyncPortOutHigh                                                   ' % 19 
    'SyncPortOutLow                                                    ' % 20
    'Unparking car                                                     ' % 21 (15)
    'Unparking car experimenter                                        ' % 22 (16)
    'Wheels of car                                                     ' % 23 (17)
    'Wheel any other moving car LF                                     ' % 24
    'Wheel any other moving car LR                                     ' % 25
    'Wheel any other moving car RF                                     ' % 26
    'Wheel any other moving car RR                                     ' % 27
    'Wheel approaching LF                                              ' % 28
    'Wheel approaching LR                                              ' % 29
    'Wheel approaching RF                                              ' % 30
    'Wheel approaching RR                                              ' % 31
    'Wheel parked LF                                                   ' % 32
    'Wheel parked LR                                                   ' % 33
    'Wheel parked RF                                                   ' % 34
    'Wheel parked RR                                                   ' % 35
    'Wheel parking/unparking - moving backward LF                      ' % 36
    'Wheel parking/unparking - moving backward LR                      ' % 37
    'Wheel parking/unparking - moving backward RF                      ' % 38
    'Wheel parking/unparking - moving backward RR                      ' % 39
    'Wheel parking/unparking - moving forward LF                       ' % 40
    'Wheel parking/unparking - moving forward LR                       ' % 41
    'Wheel parking/unparking - moving forward RF                       ' % 42
    'Wheel parking/unparking - moving forward RR                       '}; % 43)

% reduced events to 14 categories (after summing a number of categories)
Events_reduced={'Approaching car' % 2
    'Driver in approaching car' % 10
    'Parking/unparking car' % 13+15+16
    'Driver in parking/unparking car' % 5+11
    'Other moving car' % 1
    'Driver in other moving car' % 9
    'Wheel of moving or parked car' % 17
    'Back of parked car' % 3
    'Side of parked car' % 14
    'Front of parked car' % 7
    'Human outside car' % 12
    'Experimenter' % 6
    'Ground (asphalt/lines)' % 8
    'Environment'}; % 4

%% Import Tobii data
% open file and define file structure
fileID = fopen(FILE_TOBII,'r','n');
     %1              %2                             %3        %4         %5     %6-24                                                     %25
fmt='%f %*s%*s%*s%*s %s %*s%*s%*s%*s%*s%*s%*s%*s%*s %s %*s%*s %s %*s%*s  %s %*s %f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f %*s%*s%*s%*s%*s%*s %f%f%f%f%f%f%f%f%f%f %*[^\n]';
% import data exported from Tobii software
X = textscan(fileID, fmt, 'Delimiter', {'\t'}, 'HeaderLines' , 1, 'TextType', 'string', 'HeaderLines' , 1, 'ReturnOnError', false, 'EndOfLine', '\r\n');
% Participant number
temp=char(X{2});
X2(:,2)=str2num(temp(:,end-2:end)); %#ok<ST2NM> % participant number

% Transform event strings to numbers
temp=char(X{5});
tempp=NaN(size(temp,1),length(Events_tobii));
for i=1:size(temp,1) % loop over all rows
    tempp(i,:)=strcmp(temp(i,:),Events_tobii); % assign Tobii event
end
[X2(:,5),~]=find(tempp');
% Transform the other data to numbers and store in X2
for i=[1 3:4 6:size(X,2)]
    X2(:,i)=X{i};
end
%% Process raw fixations data
% mapping with chronological order
PP=[26 38 01 12 39 13 27 22 02 14 28 40 23 03 29 41 16 15 24 42 17 05 25 30 31 18 43 06 07 44 11 19 08 20 33 09 34 21 10 04 35 36 37];
temp=find(X2(:,1)==0); % look for starts of new session
flagduplicate=zeros(length(temp),1);
for i=2:length(temp)
    if temp(i)-temp(i-1)==1
        flagduplicate(i)=1;
    end
end

temp(flagduplicate==1)=[];
for i=1:length(temp)
    if i<length(temp) % if not last session
        PPnum(temp(i):temp(i+1)-1)=PP(i);
    else %
        PPnum(temp(i):size(X2,1))=PP(i);
    end
end
X2a=fillmissing(X2,'previous');

% if there is no fixation then put at -99999999
X2a(isnan(X2(:,27)),27)=-99999999;
X2a(isnan(X2(:,28)),28)=-99999999;
X2=X2a;clear X2a

AllFixationTypes=NaN(44,700);
for pp=PP % loop over participants
    Xpp=X2(PPnum==pp,:); % data for one participant
    time=Xpp(:,1);
    for i=2:length(time)
        if time(i)<=time(i-1)
            % ensure that there are no duplicates in the time value
            time(i)=time(i-1)+0.1;
        end
    end
    Xpp(:,1)=time;
    timenew=0:1:max(Xpp(:,1));
    Xpp=interp1(Xpp(:,1),Xpp,timenew,'nearest');
    for i=size(Xpp,1):-1:2
        % ensure there is only 1 event per fixation
        % (i.e., remove duplicates that occurred due to interp1)
        if Xpp(i,5)>1 && Xpp(i,5)==Xpp(i-1,5)
            Xpp(i,5)=1;
        end
    end
    Xpp(Xpp==-99999999)=NaN;
    time=timenew;
    Gyro=Xpp(:,29:31);
    Accelerometer=Xpp(:,32:34);
    % start & end samples to be removed
    R=[1:3000 length(time)-10000:length(time)];
    time(R)=[];
    time=time-time(1);
    Accelerometer(R,:)=[];
    Gyro(R,:)=[];
    Xpp(R,:)=[];
    % if sample rate = 1000 hz, Wn = 1.0 = 500 Hz, Wn = 0.001 = 0.5 Hz,
    % Wn = 0.0005 = 0.25 Hz
    [b,a]=butter(2,0.004);
    temp=filtfilt(b,a,Accelerometer(:,2)/10000);
    [~,b]=findpeaks(temp);
    
    SF(pp)=length(b)/max(time); % step frequency
    HM(pp)=mean(abs(Gyro(:,2))); % head movement speed
    
    walkingamplitude=medfilt1(abs(temp+10^-3),600);
    standingstill=find(walkingamplitude<0.5*median(walkingamplitude));
 
    WP(pp) = 100*(1-length(standingstill)./length(walkingamplitude)); % walking percentage
       
    temp=find(xlsPP==pp);
    Gender(pp)=xlsGender(temp);
    Age(pp)=xlsAge(temp);
    
    temp=find(Xpp(:,5)>1); % find all fixations
    NF(pp)=length(temp); % number of fixations for that person
    AD(pp)=mean(~isnan(Xpp(:,27))); % proportion of data that are fixations
    LF(pp)=range(Xpp(:,1))/1000;% duration of session, in seconds
    for i=2:43 % loop over AOIs
        tempi=find(Xpp(:,5)==i);
        NFt(pp,i)=length(tempi);
        SFDt(pp,i)=sum(Xpp(tempi,25)); % sum of fixation durations
    end
    temp=Xpp(Xpp(:,5)>0,5); % find fixations
    temp=temp(ismember(temp,[2 3 4 5 6 7 8 9 10 11 12 13 14 18 21 22 23:43]));  % remove other events (not fixations)
    % store all fixations per participant
    AllFixationTypes(pp,1:length(temp))=temp; % 44 x 700 matrix
    % create figure with example head yaw rate data
    if pp==18 || pp == 41
        figure(999);hold on;
        if pp==18
            subplot(1,2,1)
            set(gca,'pos',[0.06 0.11 0.4 0.86])
            title('Low head movement index')
        elseif pp==41
            subplot(1,2,2)
            set(gca,'pos',[0.58 0.11 0.4 0.86])
            title('High head movement index')
        end
        % positive is towards the left
        plot(time/1000,Gyro(:,2),'k-','Linewidth',1)
        box off;grid on
        set(gca,'ylim',[-400 400],'xlim',[0 180])
        xlabel('Time (s)')
        ylabel('Head yaw rate (deg/s)')
        fig=gcf;
        set(findall(fig,'-property','FontName'), 'FontName','Arial','Fontsize',24)       
    end
end
disp('Head movement index of participant with a low head movement index:')
disp(HM(18))
disp('Head movement index of participant with a high head movement index:')
disp(HM(41))

%% Create 44x43x43 fixation matrix with links between AOIs
FixationMatrix=zeros(44,43,43);
includedpp=find(AD>.1);
disp(nansum(NFt(includedpp,24:end))) % number of fixatons on wheels
% now that wheels sub-AOIs have been displayed, merge into 1 wheels category

NFt(:,23)=sum(NFt(:,24:43),2); % total fixation durations
SFDt(:,23)=sum(SFDt(:,24:43),2); % sum of fixation durations

for i=1:length(includedpp) % loop over participants
    for i2=1:size(AllFixationTypes,2)-1 % loop over all fixations
        Fixation1=AllFixationTypes(includedpp(i),i2);
        Fixation2=AllFixationTypes(includedpp(i),i2+1);
        if ~isnan(Fixation2)
        FixationMatrix(includedpp(i),Fixation1,Fixation2) = FixationMatrix(includedpp(i),Fixation1,Fixation2)+1; % participant, from, to
        end
    end
end
FixationMatrix(~ismember(1:44,includedpp),:,:)=NaN;

%% Process the raw annotated data into main 14 AOIs

% discard non-events
NFtr=NFt(:,[2 3 4 5 6 7 8 9 10 11 12 13 14 18 21 22 23]);
SFDtr=SFDt(:,[2 3 4 5 6 7 8 9 10 11 12 13 14 18 21 22 23]);

% merge a number of categories such that NFtr consists of 14 categories
NFtr=[NFtr(:,2) NFtr(:,10) NFtr(:,13)+NFtr(:,15)+NFtr(:,16) NFtr(:,5)+NFtr(:,11) NFtr(:,[1 9 17 3 14 7 12 6 8 4])];
SFDtr=[SFDtr(:,2) SFDtr(:,10) SFDtr(:,13)+SFDtr(:,15)+SFDtr(:,16) SFDtr(:,5)+SFDtr(:,11) SFDtr(:,[1 9 17 3 14 7 12 6 8 4])]; % sum of fixation durations (ms)
MFDtr=SFDtr./NFtr; % mean fixation duration (ms)

% normalize per participant
NFtrn=NFtr./repmat(nansum(NFtr,2),1,length(Events_reduced));
%% Figure 3. The number of transitions (NT) between AOIs 
FixationMatrixr=squeeze(nansum(FixationMatrix,1)); % sum of all 36 participants

% discard non-events              
FixationMatrixr=FixationMatrixr([2 3 4 5 6 7 8 9 10 11 12 13 14 18 21 22 23:43], ...
                                [2 3 4 5 6 7 8 9 10 11 12 13 14 18 21 22 23:43]);

% merge a number of categories such that NFtr consists of 14 categories
FixationMatrixr=[FixationMatrixr(:,2) FixationMatrixr(:,10)  ...
                 FixationMatrixr(:,13)+FixationMatrixr(:,15)+FixationMatrixr(:,16) ...
                 FixationMatrixr(:,5)+FixationMatrixr(:,11) ...
                 FixationMatrixr(:,[1 9]) ...
                 sum(FixationMatrixr(:,18:37),2) ...
                 FixationMatrixr(:,[3 14 7 12 6 8 4])];

% merge a number of categories such that NFtr consists of 14 categories
FixationMatrixr=[FixationMatrixr(2,:);FixationMatrixr(10,:); ...
                 FixationMatrixr(13,:)+FixationMatrixr(15,:)+FixationMatrixr(16,:); ... 
                 FixationMatrixr(5,:)+FixationMatrixr(11,:); ...
                 FixationMatrixr([1 9],:); ...
                 sum(FixationMatrixr(18:37,:)); ...
                 FixationMatrixr([3 14 7 12 6 8 4],:)];
figure;
h=heatmap(FixationMatrixr);
xlabel('Fixation to')
ylabel('Fixation from')
h.XDisplayLabels=Events_reduced;
h.YDisplayLabels=Events_reduced;
set(gca,'pos',[0.2 0.25 0.5 0.74]);
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',18);
%% Figure 4. The normalized number of transitions (NNT) between AOIs. 
AOIs_to_show=find(nansum(FixationMatrixr)>25);
FixationMatrixrs=FixationMatrixr(AOIs_to_show,AOIs_to_show);
% normalize based on expected link values
FixationMatrixrn=FixationMatrixrs./(sum(FixationMatrixrs,2)*sum(FixationMatrixrs,2)');
% remove diagonal
eyematrix=eye(length(AOIs_to_show));
eyematrix(eyematrix==1)=NaN;
FixationMatrixrn=FixationMatrixrn+eyematrix;
FixationMatrixrn=round(100*FixationMatrixrn./sum(nansum(FixationMatrixrn)),2);
figure;
h=heatmap(FixationMatrixrn);
xlabel('Fixation to')
ylabel('Fixation from')
h.CellLabelFormat='%0.2f';
h.XDisplayLabels=Events_reduced(AOIs_to_show);
h.YDisplayLabels=Events_reduced(AOIs_to_show);
set(gca,'pos',[0.2 0.25 0.5 0.74]);
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',18);
%% Figure with correlation coefficients
opengl hardware
[c,p,clow,chigh]=corrcoef(tiedrank([HM(includedpp), NFtrn(includedpp,:)]), 'rows', 'pairwise');
c=c(2:end,1);clow=clow(2:end,1);chigh=chigh(2:end,1);p=p(2:end,1);
figure;hold on
barh(c,'facecolor',[0 144 189]/255);
for i=1:length(c)
    plot([chigh(i) chigh(i)],[i-0.15 i+.15],'k','linewidth',1)
    plot([clow(i) clow(i)],[i-0.15 i+.15],'k','linewidth',1)
    plot([clow(i) chigh(i)],[i i],'k','linewidth',1)
end
box on
set(gca,'ytick', 1:14,'yticklabel', Events_reduced,'xlim',[-0.7 0.7], 'ylim',[.4 14.6]);
set(findall(gcf,'-property','FontName'),'FontName','Arial','Fontsize',20);
xlabel('Spearman''s correlation coefficient');
set(gca,'TickDir','in','LooseInset', [0.01 0.01 0.01 0.01]);
%% Display statistical data
disp('Total number of fixations recorded, all people')
disp(sum(sum(NFtr(AD>0,:))))
disp('Average number of fixations per participant')
disp(nanmean(sum(NFtr,2)))
disp('SD=')
disp(nanstd(sum(NFtr,2)))
disp('min=')
disp(min(sum(NFtr,2)))
disp('max=')
disp(max(sum(NFtr,2)))
disp('Number of fixations, after excluding participants with only little data')
disp(sum(sum(NFtr(includedpp,:))))
disp(nanmean(sum(NFtr(includedpp,:),2)))
disp(nanstd(sum(NFtr(includedpp,:),2)))
disp(min(sum(NFtr(includedpp,:),2)))
disp(max(sum(NFtr(includedpp,:),2)))
disp('Mean and SD of head movement index')
disp([mean(HM(includedpp)) std(HM(includedpp))])
disp('Mean and SD of walking percentage (% of time)')
disp([mean(WP(includedpp)) std(WP(includedpp))])

disp('Mean and SD of number of approaching cars')
disp([round(nanmean(nansum(Events(includedpp,1:4),2)),1) ...
      round(nanstd(nansum(Events(includedpp,1:4),2)),1)])
disp('Mean and SD of number of parking/unparking cars')
disp([round(nanmean(nansum(Events(includedpp,5:8),2)),1) ...
      round(nanstd(nansum(Events(includedpp,5:8),2)),1)])
disp('Mean and SD of number of humans outside cars')
disp([round(nanmean(nansum(Events(includedpp,9:10),2)),1) ...
      round(nanstd(nansum(Events(includedpp,9:10),2)),1)])
disp('Mean and SD of % parking spaces occupied')
disp([round(nanmean(sum(Occupancy(1:57,includedpp))),1) ...
      round(nanstd(sum(Occupancy(1:57,includedpp))),1)])
disp('Mean and SD of % parking spaces occupied (parking spaces 1-29 only)')
disp([round(mean(sum(Occupancy(1:29,includedpp))),1) ...
      round(std(sum(Occupancy(1:29,includedpp))),1)])

Occ=NaN(57,1);
for i=1:57
    Occ(i)=round((sum(Occupancy(i,includedpp),2)/length(includedpp)*100),0);
end
disp('% Occupancy per parking space across participants')
disp([(1:57)' Occ])

disp('Mean and SD of recording time, all 43 participants')
disp([round(nanmean(LF(:)),1) round(nanstd(LF(:)),1)]);
disp('Mean and SD of gaze samples, first 10 participants')
disp([round(nanmean(LF(1:10)),1) round(nanstd(LF(1:10)),1)]);
disp('Mean and SD of recording time, remaining 33 participants')
disp([round(nanmean(LF(11:44)),1) round(nanstd(LF(11:44)),1)]);

disp('Mean and SD of gaze samples, all 43 participants')
disp([round(nanmean(Samples(:))*100,1) round(nanstd(Samples(:))*100,1)]);
disp('Mean and SD of fixation samples, all 43 participants')
disp([round(nanmean(AD(:))*100,1) round(nanstd(AD(:))*100,1)]);

%% Figure 2. Boxplot of the normalized fixation count. 
figure
notBoxPlot4b(100*NFtrn(includedpp,:),1);grid on
set(gca,'TickDir', 'out', 'xticklabel', Events_reduced,'xtick',1:100, 'xlim', [.6 14.8], 'ylim', [0 45], 'LooseInset', [0.01 0.01 0.01 0.01])
xtickangle(45)
ylabel('Normalized number of fixations (%)')
h=findobj('FontName','Helvetica'); set(h,'FontSize',20, 'Fontname','Arial'); set(gca,'LooseInset', [0.01 0.01 0.01 0.01]);
disp('Percentages shown in Figure 2')
percentagesinfigure=nanmean(100*NFtrn(includedpp,:));
disp(percentagesinfigure)
disp('Percentages on driver, as reported in the text')
disp([100*percentagesinfigure(2)/sum(percentagesinfigure(1:2)) 100*percentagesinfigure(4)/sum(percentagesinfigure(3:4)) 100*percentagesinfigure(6)/sum(percentagesinfigure(5:6))])
%% Figure S1. Boxplot of the fixation count.
figure
notBoxPlot4b(NFtr(includedpp,:),1);grid on
set(gca,'TickDir', 'out', 'xticklabel', Events_reduced,'xtick',1:100, 'xlim', [.6 14.8],'ylim',[0 225], 'LooseInset', [0.01 0.01 0.01 0.01])
xtickangle(45)
ylabel('Number of fixations')
h=findobj('FontName','Helvetica'); set(h,'FontSize',20, 'Fontname','Arial'); set(gca,'LooseInset', [0.01 0.01 0.01 0.01]);
%% Figure S2. Boxplot of the total fixation duration (s).
figure
notBoxPlot4b(SFDtr(includedpp,:)/1000,1);grid on
set(gca,'TickDir', 'out', 'xticklabel', Events_reduced,'xtick',1:100, 'xlim', [.6 14.8], 'ylim',[0 32],'LooseInset', [0.01 0.01 0.01 0.01])
xtickangle(45)
ylabel('Total fixation duration (s)')
h=findobj('FontName','Helvetica'); set(h,'FontSize',20, 'Fontname','Arial'); set(gca,'LooseInset', [0.01 0.01 0.01 0.01]);
disp('Correlation between medians')
disp(corr([median(NFtr(includedpp,:))' median(NFtrn(includedpp,:))' median(SFDtr(includedpp,:))']))
%% Figure S3. Boxplot of the mean fixation duration. 
figure
notBoxPlot4b(MFDtr(includedpp,:),0);grid on
set(gca,'TickDir', 'out', 'xticklabel', Events_reduced,'xtick',1:100, 'xlim', [.6 14.8],'LooseInset', [0.01 0.01 0.01 0.01])
xtickangle(45)
ylabel('Mean fixation duration (ms)')
h=findobj('FontName','Helvetica'); set(h,'FontSize',20, 'Fontname','Arial'); set(gca,'LooseInset', [0.01 0.01 0.01 0.01]);
disp('Correlation between medians')
disp(corr([median(NFtr(includedpp,:))' median(NFtrn(includedpp,:))' median(SFDtr(includedpp,:))' median(MFDtr(includedpp,:))'],'rows','pairwise'))

%% ************************************************************************
%% Supplementary functions
%% ************************************************************************
% Reduced from a script of Rob Campbell 
% https://nl.mathworks.com/matlabcentral/fileexchange/26508-notboxplot
function notBoxPlot4b(y,roundingdigits)

x=1:size(y,2);
jitter=0.2;
hold on
for ii=1:size(y,2)
    myPlotter(x(ii),y(:,ii))
end
hold off
    function myPlotter(X,Y)
        jitScale=jitter*1.2;
        for k=1:length(X)
            thisY=Y(:,k);

            thisY=thisY(~isnan(thisY));
            thisX=repmat(X(k),1,length(thisY));
            ptch=patch([X(k)-jitScale, X(k)+jitScale, X(k)+jitScale, ...
                        X(k)-jitScale],[prctile(Y(:,k),25), ...
                        prctile(Y(:,k),25),prctile(Y(:,k),75), ...
                        prctile(Y(:,k),75)], 0);
            set(ptch,'edgecolor',[0.8 0.56 0.56],'facecolor',[1 0.7 0.7])
            plot([X(k)-jitScale,X(k)+jitScale], ...
                 [median(thisY),median(thisY)], ...
                 '-r','linewidth', 2);
            thisX=violaPoints(thisX,thisY);
            plot(thisX,thisY,'o','color',[0 0 0], ...
                 'markerfacecolor','none', 'markersize', 8, ...
                 'linewidth', 2);
            plot(X(k),nanmean(Y),'x','linewidth',4,'markersize', 20, ...
                'markeredgecolor',[0 255 100]/255)

            if roundingdigits==0
                text(X(k)+0.25,nanmean(Y),sprintf('%0.0f',nanmean(Y)))
            elseif roundingdigits==1
                if nanmean(Y)==0 && ii<=2
                    text(X(k)+0.25,2.5,sprintf('%0.1f',nanmean(Y)))
                elseif nanmean(Y)==0 && ii==7
                    text(X(k)+0.25,6.25,sprintf('%0.1f',nanmean(Y)))
                else
                    text(X(k)+0.25,nanmean(Y),sprintf('%0.1f',nanmean(Y)))
                end
            elseif roundingdigits==2
                text(X(k)+0.25,nanmean(Y),sprintf('%0.2f',nanmean(Y)))
            end
        end
        
    end
    function X=violaPoints(X,Y)
        [counts,~,bins]=histcounts(Y,10);
        inds=find(counts~=0);
        counts=counts(inds);
        Xr=X;
        for jj=1:length(inds)
            tWidth=jitter*(1-exp(-0.1*(counts(jj)-1)));
            xpoints=linspace(-tWidth*0.8,tWidth*0.8,counts(jj));
            Xr(bins==inds(jj))=xpoints;
        end
        X=X+Xr;
    end
end
